/*
 * Copyright (C) 2017 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim.exporter;

import i18n.I18N;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import javax.swing.JOptionPane;
import storybook.App;
import storybook.exim.EXIM;
import storybook.model.book.Book;
import storybook.model.book.BookParamExport;
import storybook.toolkit.LOG;
import storybook.toolkit.html.CSS;
import storybook.toolkit.html.Html;
import storybook.ui.MainFrame;
import storybook.ui.dialog.ExceptionDlg;

/**
 *
 * @author favdb
 */
public abstract class AbstractExport {

	private static final String TT = "AbstractExport";

	private static final String EXPORT_TITLE = "Export";
	public static final String F_XML = "xml", F_HTML = "html", F_CSV = "csv", F_TXT = "txt";
	public static boolean TOC_LINK = true,
			MULTI_CHAPTER = true,
			MULTI_SCENE = true,
			IS_NAV = true,
			IS_NAV_IMAGE = true;

	public BookParamExport param;
	public boolean isOpened;
	public BufferedWriter outStream;
	public MainFrame mainFrame;
	public String name;
	public static final boolean VERBOSE = true;
	public static final boolean SILENT = false;
	public Book book;
	private String htmlTitle = "";

	public AbstractExport(MainFrame mainFrame, String format) {
		this.mainFrame = mainFrame;
		book = mainFrame.getBook();
		param = book.getParamExport();
		if (param.getDirectory() == null || param.getDirectory().isEmpty()) {
			param.setDirectory(mainFrame.getH2File().getPath() + File.separator + book.getTitle());
		}
		param.setFormat(format);
		isOpened = false;
	}

	public void setHtmlTitle(String title) {
		this.htmlTitle = title;
	}

	public String getFileName(String n) {
		return param.getDirectory() + File.separator + n + "." + param.getFormat().toString();
	}

	public boolean askFileExists(String n) {
		//LOG.trace(TT + ".askExists(n=" + n + ")");
		this.name = n;
		param.setFileName(getFileName(n));
		File file = new File(param.getFileName());
		return file.exists();
	}

	public boolean openFile(String name) {
		//LOG.trace(TT + ".openFile(name=" + name + ")");
		isOpened = false;
		param = mainFrame.getBook().getParamExport();
		this.name = name;
		File file = new File(param.getDirectory());
		if (!file.exists()) {
			String newdir = EXIM.getDirExport(mainFrame);
			// create a directory
			file = new File(newdir);
			if (!file.exists()) {
				file.mkdirs();
			}
			param.setDirectory(newdir);
		}
		param.setFileName(param.getDirectory() + File.separator + name + "." + param.getFormat());
		if (!(file.exists() && file.isDirectory())) {
			JOptionPane.showMessageDialog(mainFrame,
					I18N.getMsg("export.dir.error") + "\n" + file.getAbsolutePath(),
					I18N.getMsg("export"), 1);
			return (false);
		}
		try {
			outStream = new BufferedWriter(new OutputStreamWriter(
					new FileOutputStream(param.getFileName()), "UTF-8"));
		} catch (IOException ex) {
			ExceptionDlg.show(EXPORT_TITLE, ex);
			return (false);
		}
		// create header of file for Html or Xml
		switch (param.getFormat()) {
			case "xml":
				writeHeaderXml();
				break;
			case "html":
				writeHeaderHtml();
				break;
			default:
				break;
		}
		isOpened = true;
		return (true);
	}

	public boolean writeHeaderXml() {
		//App.trace("AbstractExporter.writeHeaderXml()");
		StringBuilder str = new StringBuilder();
		str.append("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
		if (param.isXml()) {
			str.append("<book>\n");
		} else if (param.isOsbk()) {
			str.append("<storyboard title=\"").append(book.getTitle()).append("\">\n");
		} else {
			str.append("<book>\n");
		}
		try {
			outStream.write(str.toString(), 0, str.length());
			outStream.flush();
		} catch (IOException ex) {
			ExceptionDlg.show(EXPORT_TITLE, ex);
			return (false);
		}
		return (true);
	}

	/**
	 * writeHeaderHtml
	 *
	 * @return false if not OK else true
	 */
	public boolean writeHeaderHtml() {
		StringBuilder str = new StringBuilder();
		str.append(Html.DOCTYPE);
		str.append(Html.BEGIN_LANG);
		str.append(Html.HEAD);
		//str.append(Html.META_CONTENT);
		str.append(Html.META_UTF8);
		if (!htmlTitle.isEmpty()) {
			str.append(Html.intoTag("title", htmlTitle));
		} else {
			String title = mainFrame.getBook().getTitle();
			str.append(Html.intoTag("title", title));
		}
		BookParamExport paramBook = mainFrame.getBook().getParamExport();
		if (Book.getBoolean(mainFrame, Book.INFO.SCENARIO)) {
			if (paramBook.getHtmlCss().isEmpty()) {
				str.append(Html.STYLE);
				str.append(CSS.forScenario(false));
				str.append(Html.STYLE_END);
			} else {
				str.append(CSS.fromFile(paramBook.getHtmlCss(), App.getInstance().fontGetMono()));
			}
		} else {
			str.append(CSS.fromFile(paramBook.getHtmlCss(), null));
		}
		str.append(Html.TABLE_STYLE);
		str.append(Html.HEAD_END);
		str.append(Html.BODY);
		try {
			outStream.write(str.toString(), 0, str.length());
			outStream.flush();
		} catch (IOException ex) {
			ExceptionDlg.show(EXPORT_TITLE, ex);
			return false;
		}
		return true;
	}

	public void writeHtml(String str) {
		writeText(Html.intoTag("tr", str));
	}

	public void writeXml(int lev, String label, String str) {
		String tab = "    ";
		StringBuilder b = new StringBuilder();
		for (int i = 0; i < lev; i++) {
			b.append(tab);
		}
		b.append("<").append(label).append(">");
		if (!str.isEmpty()) {
			b.append(str);
			b.append("</").append(label).append(">");
		}
		b.append("\n");
		writeText(b.toString());
	}

	public void writeXmlAttribute(int lev, String label, String str) {
		String tab = "    ";
		StringBuilder b = new StringBuilder();
		for (int i = 0; i < lev; i++) {
			b.append(tab);
		}
		b.append("<").append(label).append(" ");
		if (!str.isEmpty()) {
			b.append(str);
		}
		b.append(" />\n");
		writeText(b.toString());
	}

	private String xmlTab(int lev) {
		StringBuilder b = new StringBuilder();
		for (int i = 0; i < lev; i++) {
			b.append("    ");
		}
		return (b.toString());
	}

	public String stringAttribute(int lev, String key, String val) {
		return (xmlTab(lev) + " " + key + "=\"" + val + "\"");
	}

	public String stringAttribute(int lev, String key, boolean val) {
		return (xmlTab(lev) + " " + key + "=\"" + (val ? "true" : "false") + "\"");
	}

	public String stringAttribute(int lev, String key, int val) {
		return (xmlTab(lev) + " " + key + "=\"" + val + "\"");
	}

	public String stringAttribute(int lev, String key, Long val) {
		return (xmlTab(lev) + " " + key + "=\"" + val.toString() + "\"");
	}

	public void writeText(String str) {
		if (!isOpened) {
			return;
		}
		if ("".equals(str)) {
			return;
		}
		try {
			//force conversion to UTF-8
			byte[] bytes = str.getBytes();
			String forced = new String(bytes, StandardCharsets.UTF_8);
			//write the forced String to file
			outStream.write(str, 0, forced.length());
			outStream.flush();
		} catch (IOException ex) {
			LOG.err(TT + ".writeText(" + str + ")", ex);
		}
	}

	public void closeFile(boolean verbose) {
		//LOG.trace(TT + ".closeFile(verbose=" + (verbose ? "true" : "false")+")");
		if (isOpened == false) {
			return;
		}
		switch (param.getFormat()) {
			case "osbk":
				writeText("</storyboard>\n");
				break;
			case "xml":
				writeText("</book>\n");
				break;
			case "html":
				writeText(Html.BODY_END + Html.END);
				break;
			case "txt":
			case "csv":
			default:
				break;
		}
		try {
			outStream.close();
			isOpened = false;
			if (verbose) {
				JOptionPane.showMessageDialog(mainFrame,
						I18N.getMsg("export.success", param.getFileName()),
						I18N.getMsg("export"), JOptionPane.INFORMATION_MESSAGE);
			}
		} catch (IOException ex) {
			LOG.err(TT + ".closeFile(...)", ex);
		}
	}

}
